@@ -1,6 +1,6 @@
 ;;; sb-asahi.el --- shimbun backend for asahi.com -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
 ;; Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -1483,6 +1483,11 @@ that day if it failed."
 \[\t\n ]*"
 			       nil t)
        (delete-region (match-beginning 0) (match-end 0)))
+     (goto-char (point-min))
+     (while (re-search-forward "[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)+\
+$B%"%5%R!&%3%`%H%C%W(B[$B$X%X(B]\\(?:\\(?:[\t\n ]*<[^>]+>\\)+[\t\n ]*\\|[\t\n ]*\\'\\)"
+			       nil t)
+       (replace-match "\n"))
      ;; Remove trailing garbage.
      (goto-char (point-min))
      (when (and (not (string-match "$B$f$k$f$k%U%'%_%K%s(B" from))
