@@ -1,6 +1,6 @@
 ;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
 ;; Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -173,6 +173,18 @@ and set `shimbun-SERVER-japanese-hankaku' to `never'."
 	  (const :tag "Don't convert" nil)
 	  (const :tag "Never convert" never)))
 
+(defcustom shimbun-retry-fetching nil
+  "Number of times to retry fetching the web contents of a url.
+If it is a positive number and the fetching of the contents of a url
+fails, it will be retried until it is successful or until the number
+of times to retry reaches to that number.  Note that a non-nil value
+of `shimbun-SERVER-retry-fetching' overrides this variable."
+  :group 'shimbun
+  :type '(radio (const :format "Don't retry " nil)
+		(integer :tag "Number of retries"
+			 :match (lambda (widget value) (natnump value))
+			 :value 1)))
+
 (defun shimbun-servers-list ()
   "Return a list of shimbun servers."
   (let (servers)
@@ -227,6 +239,13 @@ Non-ASCII characters `url' are escaped based on `url-coding-system'."
 	(set-buffer-multibyte t)
 	nil))))
 
+(luna-define-generic shimbun-retry-fetching (shimbun)
+  "Return the number of times to retry fetching the web contents of a url.")
+
+(luna-define-method shimbun-retry-fetching ((shimbun shimbun))
+  (or (shimbun-retry-fetching-internal shimbun)
+      shimbun-retry-fetching))
+
 (defun shimbun-fetch-url (shimbun url &optional no-cache no-decode referer)
   "Retrieve contents specified by URL for SHIMBUN.
 This function is exacly similar to `shimbun-retrieve-url', but
@@ -238,7 +257,7 @@ slot of SHIMBUN to encode URL."
 	  (if coding
 	      (cons coding w3m-coding-system-priority-list)
 	    w3m-coding-system-priority-list))
-	 (retry (shimbun-retry-fetching-internal shimbun)))
+	 (retry (shimbun-retry-fetching shimbun)))
     (setq coding (or (shimbun-url-coding-system-internal shimbun) coding))
     (save-restriction
       (narrow-to-region (point) (point))
@@ -262,6 +281,7 @@ slot of SHIMBUN to encode URL."
   "Return a real URL."
   (w3m-real-url url no-cache))
 
+(defalias 'shimbun-decode-anchor-string 'w3m-decode-anchor-string)
 (defalias 'shimbun-decode-entities 'w3m-decode-entities)
 (defalias 'shimbun-decode-entities-string 'w3m-decode-entities-string)
 (defalias 'shimbun-expand-url 'w3m-expand-url)
@@ -621,9 +641,12 @@ image parts, and returns an alist of URLs and image entities."
 	    end (match-end 1)
 	    url (or (match-string 2) (match-string 3) (match-string 4)))
       (setq url (shimbun-expand-url
-		 (if (string-match "[\t\n\f\r ]+\\'" url)
-		     (substring url 0 (match-beginning 0))
-		   url)
+		 ;; See the FIXME comment within the function definition
+		 ;; of `w3m-decode-anchor-string' in w3m.el.
+		 (shimbun-decode-anchor-string
+		  (if (string-match "[\t\n\f\r ]+\\'" url)
+		      (substring url 0 (match-beginning 0))
+		    url))
 		 base-url))
       (unless (setq img (assoc url images))
 	(with-temp-buffer
