@@ -1,6 +1,6 @@
 ;;; w3m-ems.el --- GNU Emacs stuff for emacs-w3m
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -79,6 +79,7 @@
   ;; `w3m-tab-move-right' calls the inline function `w3m-buffer-set-number'
   ;; which uses it.
   (defvar w3m-use-title-buffer-name)
+  (defvar w3m-form-input-textarea-coding-system)
   (autoload 'w3m-copy-buffer "w3m")
   (autoload 'w3m-delete-buffer "w3m")
   (autoload 'w3m-image-type "w3m")
@@ -89,7 +90,9 @@
   (unless (fboundp 'frame-current-scroll-bars)
     (defalias 'frame-current-scroll-bars 'ignore))
   (unless (fboundp 'window-fringes)
-    (defalias 'window-fringes 'ignore)))
+    (defalias 'window-fringes 'ignore))
+  (unless (fboundp 'unencodable-char-position)
+    (defalias 'unencodable-char-position 'ignore)))
 
 ;;; Coding system and charset.
 
@@ -192,6 +195,10 @@ CODING-SYSTEM, DECODER and ENCODER must be symbol."
   "Add to the buffer-local value of HOOK the function FUNCTION."
   (add-hook hook function append t))
 
+(defun w3m-remove-local-hook (hook function)
+  "Remove to the buffer-local value of HOOK the function FUNCTION."
+  (remove-hook hook function t))
+
 ;; Function which returns non-nil when the current display device can
 ;; show images inline.
 (defalias 'w3m-display-graphic-p 'display-images-p)
@@ -465,7 +472,20 @@ used together."
     ,@(unless (featurep 'gtk)
 	'((tool-bar-button-relief . global))))
   "Alist of the variables and the values controls the tool bar appearance.
-The value `global' means to use the global value of the variable."
+The value `global' means to use the global value of the variable.
+
+If you're annoyed with changing of the frame height of Emacs built for
+GTK because of the difference of the sizes between the emacs-w3m tool
+bar icons and the ones that Emacs natively uses, try customizing this
+variable or both the value of this variable and the global value of
+`tool-bar-button-margin'.  For examples:
+
+;; The case where the emacs-w3m icons are smaller than the others.
+\(setq w3m-toolbar-configurations '((tool-bar-button-margin . 5)))
+
+;; The case where the emacs-w3m icons are bigger than the others.
+\(setq w3m-toolbar-configurations '((tool-bar-button-margin . 0))
+      tool-bar-button-margin 7)"
   :group 'w3m
   :type '(repeat (cons :format "%v"
 		       (symbol :tag "Variable")
@@ -640,6 +660,17 @@ otherwise works in all the emacs-w3m buffers."
 (put 'w3m-tab-unselected-retrieving-face
      'face-alias 'w3m-tab-unselected-retrieving)
 
+(defface w3m-tab-unselected-unseen
+  '((((type x w32 mac) (class color))
+     :background "Gray70" :foreground "Gray20"
+     :box (:line-width -1 :style released-button))
+    (((class color))
+     (:background "blue" :foreground "gray60")))
+  "*Face to fontify unselected and unseen tabs."
+  :group 'w3m-face)
+;; backward-compatibility alias
+(put 'w3m-tab-unselected-unseen-face 'face-alias 'w3m-tab-unselected-unseen)
+
 (defface w3m-tab-selected
   '((((type x w32 mac) (class color))
      :background "Gray90" :foreground "black"
@@ -886,7 +917,7 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 				  start 'display nil end)))
 	    (setq len (+ (cond
 			  ((eq (car disp) 'image)
-			   (or w3m-favicon-size (frame-char-height)))
+			   (or (car w3m-favicon-size) (frame-char-height)))
 			  ((eq (car disp) 'space)
 			   (* (or (plist-get (cdr disp) :width) 1) fcw))
 			  (t
@@ -1072,7 +1103,7 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 		       1))
 	     (spinner (when w3m-process-queue
 			(w3m-make-spinner-image)))
-	     buffer title data datum process favicon keymap face icon line)
+	     buffer title data datum process unseen favicon keymap face icon line)
 	(setq w3m-tab-timer t)
 	(run-at-time 0.1 nil
 		     (lambda (buffer)
@@ -1100,6 +1131,7 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 			 0)))
 	    (push (list (eq current buffer)
 			w3m-current-process
+			(w3m-unseen-buffer-p buffer)
 			title
 			(when w3m-use-favicon w3m-favicon-image)
 			w3m-tab-map)
@@ -1111,9 +1143,10 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 	  (setq datum (pop data)
 		current (car datum)
 		process (nth 1 datum)
-		title (nth 2 datum)
-		favicon (nth 3 datum)
-		keymap (nth 4 datum)
+		unseen (nth 2 datum)
+		title (nth 3 datum)
+		favicon (nth 4 datum)
+		keymap (nth 5 datum)
 		face (list
 		      (if process
 			  (if current
@@ -1121,7 +1154,9 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 			    'w3m-tab-unselected-retrieving)
 			(if current
 			    'w3m-tab-selected
-			  'w3m-tab-unselected)))
+			  (if unseen
+			      'w3m-tab-unselected-unseen
+			    'w3m-tab-unselected))))
 		icon (when graphic
 		       (cond
 			(process
@@ -1311,6 +1346,22 @@ list."
 			    w3m-coding-system
 			    'iso-2022-7bit)))
 
+(defun w3m-form-coding-system-accept-region-p (&optional from to coding-system)
+  "Check whether `coding-system' can encode specified region."
+  (let ((from (or from (point-min)))
+	(to (or to   (point-max)))
+	(coding-system (or coding-system 
+			   w3m-form-input-textarea-coding-system)))
+    (if (fboundp 'unencodable-char-position)
+	(let ((pos (unencodable-char-position from to coding-system)))
+	  (or (not pos)
+	      (y-or-n-p (format "\"%c\" would not be accepted. Continue? "
+				(char-after pos)))))
+      (let ((select-safe-coding-system-accept-default-p nil))
+	(or (eq (select-safe-coding-system from to coding-system)
+		coding-system)
+	    (y-or-n-p "This text may cause coding-system problem. Continue? "))))))
+
 (provide 'w3m-ems)
 
 ;;; w3m-ems.el ends here
