@@ -1,6 +1,6 @@
 ;;; w3m-form.el --- Stuffs to handle <form> tag
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -71,6 +71,20 @@ Files to save text are stored in the directory specified by the
 		 (const :tag "Remove when emacs-w3m exit" t)
 		 (const :tag "No expire" nil)))
 
+(defcustom w3m-form-textarea-file-coding-system
+  (cond ((or (featurep 'un-define) 
+	     (fboundp 'utf-translate-cjk-mode))
+	 'utf-8)
+	((equal "Japanese" w3m-language)
+	 'iso-2022-7bit-ss2)
+	((w3m-find-coding-system 'utf-8)
+	 'utf-8)
+	(t
+	 'iso-2022-7bit-ss2))
+  "Coding system for textarea's backup file."
+  :group 'w3m
+  :type '(coding-system :size 0))
+
 (defcustom w3m-form-textarea-directory
   (expand-file-name ".textarea" w3m-profile-directory)
   "*Name of the directory to save the file of textarea input."
@@ -942,9 +956,13 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 (defun w3m-form-input (form id name type width maxlength value)
   (save-excursion
     (let* ((fvalue (w3m-form-get form id))
-	   (input (read-from-minibuffer (concat (upcase type) ": ") fvalue)))
-      (w3m-form-put form id name input)
-      (w3m-form-replace input))))
+	   (input (read-from-minibuffer (concat (upcase type) ": ") fvalue))
+	   (coding (w3m-form-get-coding-system (w3m-form-charlst form))))
+      (when (with-temp-buffer
+	      (insert input)
+	      (w3m-form-coding-system-accept-region-p nil nil coding))
+	(w3m-form-put form id name input)
+	(w3m-form-replace input)))))
 
 (defun w3m-form-input-password (form id name)
   (let* ((fvalue (w3m-form-get form id))
@@ -1102,20 +1120,23 @@ character."
        (setq file (concat file id ".txt"))
        (convert-standard-filename file)))))
 
-(defun w3m-form-input-textarea-save (&optional buffer file)
+(defun w3m-form-input-textarea-save (&optional buffer file no-check)
   "Save textarea buffer."
   (interactive)
   (setq buffer (or buffer (current-buffer)))
   (setq file (or file w3m-form-input-textarea-file))
   (with-current-buffer buffer
     (if (/= (buffer-size) 0)
-	(when (or w3m-form-use-textarea-backup-p
-		  (and (eq this-command 'w3m-form-input-textarea-save)
-		       (y-or-n-p "Really save this buffer? ")))
+	(when (and
+	       (or w3m-form-use-textarea-backup-p
+		   (and (eq this-command 'w3m-form-input-textarea-save)
+			(y-or-n-p "Really save this buffer? ")))
+	       (or no-check
+		   (w3m-form-coding-system-accept-region-p)))
 	  (let ((buffer-file-coding-system
-		 w3m-form-input-textarea-coding-system)
+		 w3m-form-textarea-file-coding-system)
 		(coding-system-for-write
-		 w3m-form-input-textarea-coding-system))
+		 w3m-form-textarea-file-coding-system))
 	    (write-region (point-min) (point-max) file nil 'nomsg)))
       (when w3m-form-use-textarea-backup-p
 	(when (file-exists-p file)
@@ -1137,19 +1158,20 @@ character."
 	(wincfg w3m-form-input-textarea-wincfg)
 	(file w3m-form-input-textarea-file)
 	info)
-    (w3m-form-input-textarea-save buffer file)
-    (or (one-window-p) (delete-window))
-    (kill-buffer buffer)
-    (if (not (buffer-live-p w3mbuffer))
-	(and (eq this-command 'w3m-form-input-textarea-set)
-	     (message "No current w3m buffer"))
-      (pop-to-buffer w3mbuffer)
-      (set-window-configuration wincfg)
-      (when (and form point)
-	(goto-char point)
-	(setq info (w3m-form-textarea-info))
-	(w3m-form-put form (nth 0 info) (nth 1 info) input)
-	(w3m-form-textarea-replace hseq input)))))
+    (when (w3m-form-coding-system-accept-region-p)
+      (w3m-form-input-textarea-save buffer file t)
+      (or (one-window-p) (delete-window))
+      (kill-buffer buffer)
+      (if (not (buffer-live-p w3mbuffer))
+	  (and (eq this-command 'w3m-form-input-textarea-set)
+	       (message "No current w3m buffer"))
+	(pop-to-buffer w3mbuffer)
+	(set-window-configuration wincfg)
+	(when (and form point)
+	  (goto-char point)
+	  (setq info (w3m-form-textarea-info))
+	  (w3m-form-put form (nth 0 info) (nth 1 info) input)
+	  (w3m-form-textarea-replace hseq input))))))
 
 (defun w3m-form-input-textarea-exit ()
   "Exit from w3m form textarea mode."
@@ -1255,8 +1277,8 @@ textarea")))
     (unless (consp buffer)
       (when (and backup-p (file-exists-p file) (file-readable-p file))
 	(with-temp-buffer
-	  (let ((buffer-file-coding-system coding)
-		(coding-system-for-read coding))
+	  (let ((buffer-file-coding-system w3m-form-textarea-file-coding-system)
+		(coding-system-for-read w3m-form-textarea-file-coding-system))
 	    (insert-file-contents file))
 	  (let ((before (buffer-string)))
 	    (when (unless (w3m-form-textarea-same-check value before)
@@ -1785,7 +1807,8 @@ textarea")))
 			(if new-session
 			    'w3m-goto-url-new-session
 			  'w3m-goto-url))
-		      (concat url "?" (w3m-form-make-form-data form))))
+		      (concat (w3m-url-strip-query url) 
+			      "?" (w3m-form-make-form-data form))))
 	    (t
 	     (w3m-message "This form's method has not been supported: %s"
 			  (let (print-level print-length)
