@@ -1,6 +1,6 @@
 ;;; w3m-xmas.el --- XEmacs stuff for emacs-w3m
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -992,6 +992,17 @@ necessarily solve the problem completely."
 	(set-window-hscroll window (- (point) (point-at-bol))))))
   )
 
+(defun w3m-form-coding-system-accept-region-p (&optional from to coding-system)
+  "Check whether `coding-system' can encode specified region."
+  (let ((string (buffer-substring (or from (point-min)) 
+				  (or to   (point-max)))))
+    (unless (string= (decode-coding-string
+		      (encode-coding-string string coding-system)
+		      coding-system)
+		     string)
+      (message "Warning: this text may cause coding-system problem."))
+    t))
+
 (provide 'w3m-xmas)
 
 ;;; w3m-xmas.el ends here
